import { Component, ChangeDetectionStrategy, Input, ViewChild, EventEmitter, ViewEncapsulation, Output, NgModule, NO_ERRORS_SCHEMA } from '@angular/core';
import { NativeScriptModule } from '@nativescript/angular';
import { Page, isIOS } from '@nativescript/core';
import { map } from 'rxjs/operators';
import { StoreDevtools } from '@ngrx/store-devtools';

const TOGGLE_BTN_HEIGHT = 40;
class NSDockMonitor {
    constructor(page) {
        this.page = page;
        this.screenCover = 0.5;
        this.shown = false;
        this.toggleLength = TOGGLE_BTN_HEIGHT;
    }
    ngAfterViewInit() {
        this.toggleBtn = this.toggleBtnEl.nativeElement;
    }
    dockLoaded(args) {
        this.dock = args.object;
        setTimeout(() => this.setup(), 100);
    }
    setup() {
        const height = this.dock.getActualSize().height;
        console.log("Dock height: " + height);
        this.offsetHidden = height - this.toggleLength;
        this.offsetShown = height * this.screenCover - this.toggleLength;
        this.dock.height = this.offsetHidden - this.offsetShown + this.toggleLength;
        this.dock.animate({ translate: { x: 0, y: this.offsetHidden } });
    }
    toggleShown() {
        this.shown = !this.shown;
        if (this.shown) {
            this.toggleBtn.animate({ rotate: 180 });
            this.dock.animate({ translate: { x: 0, y: this.offsetShown } });
        }
        else {
            this.toggleBtn.animate({ rotate: 0 });
            this.dock.animate({ translate: { x: 0, y: this.offsetHidden } });
        }
    }
}
NSDockMonitor.decorators = [
    { type: Component, args: [{
                selector: 'ns-dock-monitor',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: `
    <grid-layout rows="auto *" #dock class="dock" rowSpan="100" colSpan="100" translateY="1000" (loaded)="dockLoaded($event)">
      <label text="^" (tap)="toggleShown()" #toggle [width]="toggleLength" [height]="toggleLength" class="toggle"></label>
      <grid-layout row="1">
        <ns-log-monitor></ns-log-monitor>
      </grid-layout>
    </grid-layout>
  `,
                styles: [`
    .toggle {
        color: white;
        border-radius:` + TOGGLE_BTN_HEIGHT / 2 + `;
        border-width: 2;
        border-color: #69737D;
        background-color: #2A2F3A;
        text-align: center;
        font-family: monospace;
        font-size: 32;
        font-weight: bold;
    }
    .dock {
        vertical-align: top;
    }
  `]
            },] }
];
NSDockMonitor.ctorParameters = () => [
    { type: Page }
];
NSDockMonitor.propDecorators = {
    screenCover: [{ type: Input }],
    toggleBtnEl: [{ type: ViewChild, args: ["toggle", { static: false },] }],
    dockEl: [{ type: ViewChild, args: ["dock", { static: false },] }]
};

class NSLogMonitor {
    constructor(devtools) {
        this.devtools = devtools;
        this.canRevert$ = devtools.liftedState.pipe(map(s => !(s.computedStates.length > 1)));
        this.canSweep$ = devtools.liftedState.pipe(map(s => !(s.skippedActionIds.length > 0)));
        this.canCommit$ = devtools.liftedState.pipe(map(s => !(s.computedStates.length > 1)));
        this.items$ = devtools.liftedState.pipe(map(({ actionsById, skippedActionIds, stagedActionIds, computedStates }) => {
            const actions = [];
            for (let i = 0; i < stagedActionIds.length; i++) {
                const actionId = stagedActionIds[i];
                const action = actionsById[actionId].action;
                const { state, error } = computedStates[i];
                let previousState;
                if (i > 0) {
                    previousState = computedStates[i - 1].state;
                }
                actions.push({
                    key: actionId,
                    collapsed: skippedActionIds.indexOf(actionId) > -1,
                    action,
                    actionId,
                    state,
                    previousState,
                    error
                });
            }
            return actions.slice(1);
        }));
    }
    handleToggle(id) {
        this.devtools.toggleAction(id);
    }
    handleReset() {
        this.devtools.reset();
    }
    handleRollback() {
        this.devtools.rollback();
    }
    handleSweep() {
        this.devtools.sweep();
    }
    handleCommit() {
        this.devtools.commit();
    }
}
NSLogMonitor.decorators = [
    { type: Component, args: [{
                selector: 'ns-log-monitor',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: `
    <grid-layout rows="auto *" class="container">
      <stack-layout orientation="horizontal" class="toolbar">
        <log-monitor-button text="Reset" (action)="handleReset()">
        </log-monitor-button>

        <log-monitor-button text="Revert" (action)="handleRollback()" [disabled]="canRevert$ | async">
        </log-monitor-button>

        <log-monitor-button text="Sweep"(action)="handleSweep()" [disabled]="canSweep$ | async">
        </log-monitor-button>

        <log-monitor-button text="Commit" (action)="handleCommit()" [disabled]="canCommit$ | async">
        </log-monitor-button>
      </stack-layout>

      <scroll-view row="1">
        <stack-layout class="items">
          <log-monitor-entry
            *ngFor="let item of (items$ | async); let even = even"
            [even]="even"
            [item]="item"
            (toggle)="handleToggle($event.id)">
          </log-monitor-entry>
        </stack-layout>
      </scroll-view>
    </grid-layout>
  `,
                styles: [`
    .container {
      background-color: #2A2F3A;
      font-family: monospace;
    }

    .toolbar {
      horizontal-align: center;
      height: 32;
    }

    .items {
      background-color: #69737D;
      horizontal-align: stretch;
    }
  `]
            },] }
];
NSLogMonitor.ctorParameters = () => [
    { type: StoreDevtools }
];

let logSuffixCount = 0;
class LogMonitorEntry {
    constructor() {
        this.toggle = new EventEmitter();
    }
    handleToggle() {
        this.toggle.next({ id: this.item.actionId });
    }
    logPayload() {
        console.log("ACTION: " + JSON.stringify(this.item.action) + this.logSuffix());
    }
    logState() {
        console.log("STATE: " + JSON.stringify(this.item.state) + this.logSuffix());
    }
    logSuffix() {
        // IOS console stacks same messages. Add an " " suffix on every second message to prevent that.
        if (isIOS) {
            return (logSuffixCount++ % 2) ? "" : " ";
        }
        else {
            return "";
        }
    }
}
LogMonitorEntry.decorators = [
    { type: Component, args: [{
                selector: 'log-monitor-entry',
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.Emulated,
                template: `
    <grid-layout columns="* auto" class="container" [class.even]="even">
      <grid-layout (tap)="handleToggle()">
        <label [text]="item.action.type"
              class="title-bar"
              [class.collapsed]="item.collapsed"></label>
      </grid-layout>

      <stack-layout col="1" *ngIf="!item.collapsed" orientation="horizontal">
        <log-monitor-button (action)="logPayload()" text="Log Payload">
        </log-monitor-button>

        <log-monitor-button (action)="logState()" text="Log State">
        </log-monitor-button>
      </stack-layout>
    </grid-layout>
  `,
                styles: [`
    .container {
      horizontal-align: stretch;
      height: 32;
    }
    .title-bar {
      text-align: left;
      horizontal-align: stretch;
      vertical-align: center;
      margin: 6 10;
      font-family: monosapce;
      color: #FFFFFF;
      font-size: 12;
      font-weight: bold;
    }
    .collapsed{
      text-decoration: line-through;
      font-style: italic;
      opacity: 0.5;
    }
    .even {
      background-color: #7A8590;
    }
  `]
            },] }
];
LogMonitorEntry.propDecorators = {
    item: [{ type: Input }],
    even: [{ type: Input }],
    toggle: [{ type: Output }]
};

class LogMonitorButton {
    constructor() {
        this.action = new EventEmitter();
        this.text = "";
    }
    handleAction() {
        if (!this.disabled) {
            this.action.next({});
        }
    }
}
LogMonitorButton.decorators = [
    { type: Component, args: [{
                selector: 'log-monitor-button',
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.Emulated,
                template: `
  <button [text]="text" (tap)="handleAction()" [class.disabled]="disabled"></button>`,
                styles: [`
     button {
      font-family: monospace;
      font-weight: bold;
      border-radius: 3;
      margin: 5 3;
      padding: 3 5;
      font-size: 10;
      color: white;
      background-color: #4F5A65;
    }

    .disabled{
      opacity: 0.2;
      background-color: transparent;
    }
  `]
            },] }
];
LogMonitorButton.propDecorators = {
    disabled: [{ type: Input }],
    action: [{ type: Output }],
    text: [{ type: Input }]
};

class NativeScriptDevToolsMonitors {
}
NativeScriptDevToolsMonitors.decorators = [
    { type: NgModule, args: [{
                declarations: [LogMonitorEntry, LogMonitorButton, NSLogMonitor, NSDockMonitor],
                imports: [NativeScriptModule],
                exports: [NSDockMonitor],
                schemas: [NO_ERRORS_SCHEMA],
            },] }
];

/*
 * Public API Surface of ngrx-devtools-nativescript
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NativeScriptDevToolsMonitors, LogMonitorEntry as ɵa, LogMonitorButton as ɵb, NSLogMonitor as ɵc, NSDockMonitor as ɵd };
//# sourceMappingURL=ngrx-devtools-nativescript.js.map
